@@ -29,11 +29,13 @@ sub run {
     $self->log->info(
         'run source ' . ( $self->name || $self->path ) . ': ' . $self->source );
 
-    $self->_update_version( $self->name || $self->just_name( $self->path ),
-        $self->version );
+    $self->name( $self->just_name( $self->path ) )       unless $self->name;
+    $self->version( $self->just_version( $self->path ) ) unless $self->version;
+    $self->log->info( 'run source ' . $self->name . ': ' . $self->source );
 
-    $self->_update_url( $self->name || $self->just_name( $self->path ),
-        'directory:' . $self->source )
+    $self->_update_version( $self->name, $self->version );
+
+    $self->_update_url( $self->name, 'directory:' . $self->source )
       unless $self->{_no_update_url};
 
     my $newer = $self->_cmd;    # if we really get something new
@@ -105,7 +107,7 @@ sunnavy C<< <sunnavy@bestpractical.com> >>
 
 =head1 LICENCE AND COPYRIGHT
 
-Copyright 2007 Best Practical Solutions LLC.
+Copyright 2007-2009 Best Practical Solutions LLC.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
