@@ -53,15 +53,21 @@ sub _run {
     my @cmds;
     push @cmds,
       [
-        'svk', 'co', $self->source,
+        $ENV{'SHIPWRIGHT_SVK'},
+        'co',
+        $self->source,
         catdir( $self->download_directory, $self->name ),
         $self->version ? ( '-r', $self->version ) : ()
       ];
     push @cmds,
-      [ 'svk', 'co', '-d', catdir( $self->download_directory, $self->name ), ];
+      [
+        $ENV{'SHIPWRIGHT_SVK'}, 'co', '-d',
+        catdir( $self->download_directory, $self->name ),
+      ];
 
     unless ( $self->version ) {
-        my ($out) = Shipwright::Util->run( [ 'svk', 'info', $self->source, ] );
+        my ($out) = Shipwright::Util->run(
+            [ $ENV{'SHIPWRIGHT_SVK'}, 'info', $self->source, ] );
 
         if ( $out =~ /^Revision: (\d+)/m ) {
             $self->version($1);
@@ -105,7 +111,7 @@ sunnavy  C<< <sunnavy@bestpractical.com> >>
 
 =head1 LICENCE AND COPYRIGHT
 
-Copyright 2007 Best Practical Solutions.
+Copyright 2007-2009 Best Practical Solutions.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
